# EjemploPWM.s - Encendemos el LED RGB con PWM
# Cada vez que accionamos el pulsador, cambiamos de intensidad

.syntax unified
.cpu cortex-m3
.text
.align  2
.thumb
.thumb_func

.extern pmc_enable_periph_clk
.extern delay
.extern colores      @ Dirección de la variable compartida 'colores'
.global PWMsetup    @ Para que la función 'PWMsetup'
.type PWMsetup, %function    @   sea accesible desde los módulos externos
.global EjemploPWM    @ Para que la función 'EjemploPWM'
.type EjemploPWM, %function    @   sea accesible desde los módulos externos

# Direcciones base, offsets y máscaras de los registros de los PIO
.equ PIOB,    0x400E1000    @ Dir. base del PIO B
.equ PIOC,    0x400E1200    @ Dir. base del PIO C

.equ PIO_PER,    0x000      @ Offset Pio Enable Reg
.equ PIO_PDR,    0x004      @ Offset Pio Disable Reg
.equ PIO_PSR,    0x008      @ Offset Pio Status Reg
.equ PIO_OER,    0x010      @ Offset Output Enable Reg
.equ PIO_ODR,     0x014      @ Offset Output Disable Reg
.equ PIO_IFER,    0x020      @ Offset Glitch Input Filter Enable Reg
.equ PIO_SODR,    0x030       @ Offset Set Output Data Reg
.equ PIO_CODR,    0x034       @ Offset Clear Output Data Reg
.equ PIO_PDSR,    0x03C      @ Offset Pin Data Status Reg
.equ PIO_IDR,    0x044      @ Offset Interrupt Disable Reg
.equ PIO_PUDR,    0x060      @ Offset Pull Up Disable Reg
.equ PIO_PUER,    0x064      @ Offset Pull Up Enable Reg
.equ PIO_ABSR,    0x070      @ Offset A B Select Register
.equ PIO_DIFSR,    0x084      @ Offset Debouncing Input Filter Select Register (puerto C)
.equ PIO_WPMR,    0x0E4      @ Offset PIO Write Protect Mode Register (puerto C)

.equ ID_PIOB,    0x00C      @ ID del PIOB para el reloj
.equ RMSK,    0x00400000    @ Es el pin 8, bit 22 de PIOC. Color ROJO. Señal PWM PWML5
.equ GMSK,    0x00800000    @ Es el pin 7, bit 23 de PIOC. Color VERDE. Señal PWM PWML6
.equ BMSK,    0x01000000    @ Es el pin 6, bit 24 de PIOC. Color AZUL. Señal PWM PWML7
.equ ALLMSK,    RMSK+GMSK+BMSK    @ Máscara conjunta de los tres LEDs
.equ PULMSK,    0x08000000    @ Es el pin 13, bit 27 de PIOB. Pulsador

#  Direcciones base, offsets y máscaras de los registros del reloj del sistema
.equ PMC_BASE,    0x400E0600    @ Dirección base del PMC
.equ PMC_WPMR,    0x0E4      @ Offset del PMC Write Protect Mode Register
.equ PMC_PCER1,    0x100      @ Offset del Peripheral Clock Enable Register
.equ PMC_PCSR1,    0x108      @ Offset del Peripheral Clock Status Register
.equ PWM_MASK,    0x00000010    @ Máscara para activar el reloj del PWM
.equ FRE_MASK,    0x504D4300    @ Máscara para desproteger la escritura del PMC_PCER1
.equ BLQ_MASK,    0x504D4301    @ Máscara para bloquear la escritura del PMC_PCER1

# Dirección base y offsets de los registros del PWM
.equ PWM_BASE,    0x40094000    @ Dirección base de E/S del PWM
.equ PWM_CLK,    0x000      @ Offset PWM Clock Register
.equ PWM_ENA,    0x004      @ Offset PWM Enable Register
.equ PWM_SR,    0x00C      @ Offset PWM Status Register
.equ PWM_WPCR,    0x0E4      @ Offset PWM Write Protect Control Register
.equ PWM_WPSR,    0x0E8      @ Offset PWM Write Protect Status Register
.equ PWM_CMR5,    0x2A0      @ Offset PWM Channel Mode Register (channel 5 - pin 22 del PIO C)
.equ PWM_CMR6,    0x2C0      @ Offset PWM Channel Mode Register (channel 6 - pin 23 del PIO C)
.equ PWM_CMR7,    0x2E0      @ Offset PWM Channel Mode Register (channel 7 - pin 24 del PIO C)

.equ PWM_CDTY5,    0x2A4      @ Offset PWM Channel Duty Cycle Register (channel 5)
.equ PWM_CDTY6,    0x2C4      @ Offset PWM Channel Duty Cycle Register (channel 6)
.equ PWM_CDTY7,    0x2E4      @ Offset PWM Channel Duty Cycle Register (channel 7)

.equ PWM_CPRD5,    0x2AC      @ Offset PWM Channel Period Register (channel 5)
.equ PWM_CPRD6,    0x2CC      @ Offset PWM Channel Period Register (channel 6)
.equ PWM_CPRD7,    0x2EC      @ Offset PWM Channel Period Register (channel 7)

# Valores de configuración a escribir en los registros del PWM
.equ CMR,    0x0000000B    @ Configura el generador PWM (CLKA)
.equ CPRD,    0x000000FF    @ Configura el periodo del contador en 255 ciclos
.equ CLK,    0x00000180    @ Configura los generadores A y B (MCLK/2/128)
.equ ENA_MSK,    0x000000E0    @ Máscara para activar el canal PWM seleccionado (5, 6 y 7)

# void PWMsetup(void)
# Configuramos el pin como entrada, con pull-up y debouncing
# No acepta parámetros ni devuelve resultado por valor
.align 2
.thumb_func
PWMsetup:
   #      USO DE REGISTROS
   # r0 - Valores a escribir
   # r1 - Direcciones base de los periféricos a configurar
   # r2 - Para aplicar máscaras
   # Resto de registros no usados
    push   {lr}        @ Apilamos retorno
   
   # Configuración de los pines de los tres LEDs como salidas y asignados al PWM
   ldr     r1, =PIOC      @ Dirección base del controlador PIOC
   ldr    r0, =ALLMSK      @ Vamos a configurar los tres LED del RGB
   str    r0, [r1, #PIO_SODR]    @ Salida a uno para apagar el LED
   str    r0, [r1, #PIO_PUDR]    @ Desactivamos el pull-up que no hace falta
   str    r0, [r1, #PIO_OER]    @ Configuramos el pin como salida
   str    r0, [r1, #PIO_PER]    @ Habilitamos el bit en el PIO
   str    r0, [r1, #PIO_ABSR]    @ Asignamos a los pines 22, 23 y 24 el periférico B (PWM)   
   str    r0, [r1, #PIO_PDR]    @ Ponemos los pines 22, 23 y 24 bajo el control del periférico
   
   # Configuración del pin del pulsador como entrada con pullup
   ldr    r0, =ID_PIOB      @ Identificador de reloj del Controlador PIOB
   bl    pmc_enable_periph_clk    @ Habilitamos el reloj para la entrada
   ldr     r1, =PIOB      @ Direccion base del Controlador PIOB
   ldr    r0, =PULMSK      @ Vamos a configurar el pulsador
   str    r0, [r1, #PIO_IDR]    @ Deshabilitamos las interrupciones asociadas al pin
   str    r0, [r1, #PIO_PUER]    @ Activamos el pull-up
   str    r0, [r1, #PIO_DIFSR]    @ Seleccionamos el debouncing
   str    r0, [r1, #PIO_IFER]    @ Activamos el filtro
   str    r0, [r1, #PIO_ODR]    @ La ponemos como entrada
   str    r0, [r1, #PIO_PER]    @ Habilitamos el bit en el PIO
   
   # Activación de la señal de reloj del generador PWM en el PMC
   ldr     r1, =PMC_BASE      @ Dirección base del controlador PMC
   ldr     r0, [r1, #PMC_PCSR1]    @ Lee contenido del registro de estado
   ldr     r2, =PWM_MASK      @ Máscara de activación del reloj del PWM
   orr     r0, r2        @ Activa bit del reloj del PWM
   str     r0, [r1, #PMC_PCER1]    @ Escribe el valor de activación
   
   # Configuración de los parámetros del PWM
   ldr     r1, =PWM_BASE      @ Dirección base del controlador PWM

   ldr     r0, =CLK      @ Valor de configuración de los generadores A y B (MCLK/2/128)
   str     r0, [r1, #PWM_CLK]    @ Configuración del registro PWM_CLK. Reloj
   
   ldr     r0, =CMR      @ Valor de configuración del generador PWM (CLKA)
   str     r0, [r1, #PWM_CMR5]    @ Configuración del registro PWM_CMR del LED rojo
   str     r0, [r1, #PWM_CMR6]    @ Configuración del registro PWM_CMR del LED verde
   str     r0, [r1, #PWM_CMR7]       @ Configuración del registro PWM_CMR del LED azul
      
   ldr    r0, =ENA_MSK      @ Máscara de activación de los tres LEDs
   str    r0, [r1, #PWM_ENA]    @ Activa el PWM en los pines de salida indicados   
      
   ldr     r0, =CPRD      @ Valor de configuración de cuentas por ciclo
   str     r0, [r1, #PWM_CPRD5]    @ Configuración cuentas por ciclo del LED rojo
   str     r0, [r1, #PWM_CPRD6]    @ Configuración cuentas por ciclo del LED verde
   str     r0, [r1, #PWM_CPRD7]       @ Configuración cuentas por ciclo del LED azul
      
   pop    {pc}          @ Retorna

# void EjemploPWM(int numcolores)
# Acepta un parámetro:
# r0 = numcolores: Número de elementos de que consta la matriz de combinaciones de colores
# No devuelve resultado
.align 2
.thumb_func
EjemploPWM:
   #      USO DE REGISTROS
   # r0 - Máscara del pulsador y retardo de delay
   # r1 - Direcciones base de los dispositivos empleados
   # r2 - Dirección base de la matriz de combinaciones de colores
   # r3 - Contador para recorrer matriz de combinaciones de colores
   # r4 - Número de combinaciones de colores (parámetro 'numcolores')
   # r5 - Intensidad luminosa del LED rojo. De todos en la inicialización
   # r6 - Intensidad luminosa del LED verde
   # r7 - Intensidad luminosa del LED azul
   push   {r4-r7, lr}      @ Salvaguardamos de r4 a r7
   
   mov    r4, r0        @ Número de colores (parámetro de la función)

   # Inicialmente, iluminamos el LED RGB en blanco
   ldr     r1, =PWM_BASE      @ Dirección base del controlador PWM   
   mov     r5, #0        @ Con duty-cicle 0 intensidad máxima
   str     r5, [r1, #PWM_CDTY5]    @ Configuración del registro PWM_CDTY5. Intensidad del LED rojo
   str     r5, [r1, #PWM_CDTY6]    @ Configuración del registro PWM_CDTY5. Intensidad del LED verde
   str     r5, [r1, #PWM_CDTY7]    @ Configuración del registro PWM_CDTY5. Intensidad del LED azul
   
inicio:
   # Leemos el estado del pulsador en el PIOB
   ldr    r1, =PIOB      @ Dirección base de E/S del PIOB
   ldr    r0, =PULMSK      @ Máscara del pulsador
   ldr    r2, [r1, #PIO_PDSR]    @ Leemos el registro de E/S del puerto B
   ands    r2, r0        @ Aplicamos la máscara del pulsador
   
   # Esperamos a que se pulse el pulsador
   bne    inicio        @ Si no está pulsado seguimos esperando
   
   # Inicializamos los registros
   ldr     r1, =PWM_BASE      @ Dirección base de E/S del controlador PWM
   ldr    r2, =colores      @ Matriz que contiene las combinaciones de color
   mov    r3, #3        @ Para multiplicar por 3
   mul    r4, r4, r3      @ Multiplicamos número de colores por 3
   mov    r3, #0        @ Inicializamos contador   
   
ciclo:
   mov    r0, #50        @ Retardo
   push    {r1-r3}        @ La función podría modificar r0-r3
   bl    delay        @ Llamamos a delay
   pop    {r1-r3}        @ Recuperamos valores de r1-r3

   ldrb    r5, [r2, r3]      @ Cargamos la intensidad del LED rojo
   add    r3, #1        @ Pasamos al siguiente color
   ldrb    r6, [r2, r3]      @ Cargamos la intensidad del LED verde
   add    r3, #1        @ Pasamos al siguiente color
   ldrb    r7, [r2, r3]      @ Cargamos la intensidad del LED azul
   add    r3, #1        @ Pasamos al siguiente color
   
   cmp    r3, r4        @ Comparamos el valor actual y el máximo
   blt    cambia        @ Si es menor, podemos actualizarlo
   
   b    finpwm        @ Si es mayor o igual, regresamos
   
cambia:
   str     r5, [r1, #PWM_CDTY5]    @ Escribimos la intensidad del LED rojo
   str     r6, [r1, #PWM_CDTY6]    @ Escribimos la intensidad del LED verde
   str     r7, [r1, #PWM_CDTY7]    @ Escribimos la intensidad del LED azul
   b    ciclo        @ Pasamos a la siguiente combinación
   
finpwm:
   pop    {r4-r7, pc}       @ Desapila y retorna

.end

