// PWM_inc_cdty.s - Implementa la rutina PWM_inc_cdty

// Directivas para el ensamblador
// ------------------------------
.syntax unified
.cpu cortex-m3

// Declaración de funciones externas y exportadas
// ----------------------------------------------
.global PWM_inc_cdty               @ Para que la función 'PWM_inc_cdty' sea
.type PWM_inc_cdty, %function      @   accesible desde otros módulos

// Declaración de constantes
// -------------------------
// Dirección base y offsets de los registros del PWM
.equ PWM_BASE,  0x40094000  @ Dirección base de E/S del PWM
.equ PWM_CDTY5, 0x2A4       @ Offset PWM Channel Duty Cycle Register (channel 5)
.equ PWM_CPRD5, 0x2AC       @ Offset PWM Channel Period Register (channel 5)


// Comienzo de la zona de código
// -----------------------------
.text

/*
   Subrutina 'PWM_inc_cdty'
     Incrementa el ciclo de trabajo en 1/CPRD, incrementando el valor actual del
     registro CDTY. Cuando CDTY/CPRD llega a 1, reinicia CDTY a 0.
   Parámetros de entrada:
     No tiene.
   Parámetro de salida:
     El contenido del registro PWM_CDTY5 (se almacena en la dirección dada por r0)
     El contenido del registro PWM_CPRD5 (se almacena en la dirección dada por r1)
*/
.thumb_func
PWM_inc_cdty:
  push  {lr}                 @ Apila LR

  // Lee registros del PWM
  ldr   r0, =PWM_BASE        @ r0 <- dirección base del controlador PWM
  ldr   r1, [r0, #PWM_CDTY5] @ r1 <- PWM_CDTY (Duty cycle)
  ldr   r2, [r0, #PWM_CPRD5] @ r2 <- PWM_CPRD (Period)

  // Incrementa el ciclo de trabajo en CPRD/4
  lsr   r3, r2, #2           @ r3 <- r2 >> 2
  add   r1, r1, r3           @ r1 <- Ciclo de trabajo actualizado (+ CPRD/4)
  
  // Comprueba que no se haya superado a CPRD
  cmp   r1, r2
  ble   cont
  mov   r1, #0               @ r1 <- 0 si el ciclo era mayor que el periodo
cont:

  // Actualiza el registro PWM_CDTY
  str   r1, [r0, #PWM_CDTY5] @ Actualiza el registro PWM_CDTY con el nuevo valor

  // Retorna
  pop   {pc}                 @ Retorna (PC <- LR)
.end

